// eslint-disable-next-line object-curly-newline
import { src, dest, watch, lastRun, series, parallel } from 'gulp';
import babel from 'gulp-babel';
import chmod from 'gulp-chmod';
import less from 'gulp-less';
import del from 'del';
import scp2 from 'gulp-scp3';
import replace from 'gulp-replace';
import eslint from 'gulp-eslint';
import gulpif from 'gulp-if';
import webpack from 'webpack-stream';
import webpackConfig from './webpack.config.js';
import babelConfig from './babel.config.js';

let webpackScripts;
let copyScripts;
let packageJson;

/* output folder */
const outputDir = 'output'; // from where we ZIP for patch and/or upload into finesse server
const tempDir = 'temp'; // optional temp folder used to store files that don't get put into output folder

const scpConfig = {
  host: '',
  username: '3rdpartygadget',
  password: 'J4m1pJ4m1p',
  dest: ''
};

const paths = {
  styles: {
    src: '*.less',
    dest: outputDir
  },
  scripts: {
    src: [],
    dest: outputDir,
    temp: tempDir
  },
  assets: {
    src: [
      './*.css',
      './*.xml',
      './*.png',
    ],
    dest: outputDir
  },
};

/* update build version in file */
/* if in watch mode, then add current time as well hh:mm:ss */
function updateBuildVersion(prmMatch, val1) { // eslint-disable-line no-unused-vars
  const tme = `${new Date().toTimeString().split(' ')[0]}`;
  if (process.env.IS_GULP_WATCH === 'true') {
    // in watch mode, so update version with current date as well
    return `var gBuildVersion = 'v${packageJson.version} @ ${tme}';`;
  }
  // normal build so update build version number and add time as a comment at the end
  return `var gBuildVersion = 'v${packageJson.version}'; // ${tme}`;
}

/* update build date & time with current date & time */
function updateBuildTime(prmMatch, val1) { // eslint-disable-line no-unused-vars
  // no concern for pretty formatting as it's not displayed except in console log
  return `var gBuildDateTime = '${new Date().toGMTString()}';`;
}

/*
  for every typescript file OR javascript file that needs transpiling from ES6
  pipe through the following as a Vinyl file (i.e. a stream).
  Files aren't written to disk under any circumstances until they hit a dest()
  which means some of the output folder settings in webpack are not used.
    • Update the build version & datetime tags (only in watch mode)
    • Make writeable
    • Run through Babel to transpile and convert TS to JS
    • If the script is the master script to run through webpack
      > run it through webpack
      > save to "output" folder on disk
    • If the script is required to be copied after transpilation to output folder (not for those already bundled)
      > save to "output" folder on disk
    • Write everything else to "temp" folder - NOT sure this is actually necessary but will leave it in as not sure if it's needed in order to bundle
*/
function scripts() {
  return src(paths.scripts.src)
    .pipe(replace(/const gBuildVersion = '(.*)'.*;/g, updateBuildVersion))
    .pipe(replace(/const gBuildDateTime = '(.*)'.*;/g, updateBuildTime))
    .pipe(chmod(666))
    .pipe(babel(babelConfig))
    /* following 2 lines will perform webpack on any required file and output result into "output" folder */
    .pipe(gulpif((file) => webpackScripts.includes(file.relative), webpack(webpackConfig)))
    .pipe(gulpif((file) => webpackScripts.includes(file.relative), replace(/var gBuildVersion = '(.*)'.*;/g, updateBuildVersion)))
    .pipe(gulpif((file) => webpackScripts.includes(file.relative), replace(/var gBuildDateTime = '(.*)'.*;/g, updateBuildTime)))
    .pipe(gulpif((file) => webpackScripts.includes(file.relative), dest(paths.scripts.dest)))
    /* following 1 line will also copy the file (or output from babel) to the "output" folder */
    .pipe(gulpif((file) => copyScripts.includes(file.relative), dest(paths.scripts.dest)))
    /* always send all files to the "temp" folder (irrespective of other places they're sent) so as to keep "output" folder clean of temp files */
    .pipe(dest(paths.scripts.temp)); /* don't actually need to write any files - they'll just vapourise without writing */
}

function lint() {
  return src(paths.scripts.src)
    // eslint() attaches the lint output to the "eslint" property
    // of the file object so it can be used by other modules.
    .pipe(eslint())
    // eslint.format() outputs the lint results to the console.
    // Alternatively use eslint.formatEach() (see Docs).
    .pipe(eslint.format())
    // To have the process exit with an error code (1) on
    // lint error, return the stream and pipe to failAfterError last.
    .pipe(eslint.failAfterError());
}

/* ssh copy all files in dest/ --> Finesse 10.x server */
function scp10() {
  return src(`${outputDir}/*`)
    .pipe(scp2({ ...scpConfig, host: 'jnofin105a' }))
    .on('error', (err) => {
      console.log(err);
    });
}

/* ssh copy all files in dest/ --> Finesse 11.x server */
function scp11() {
  return src(`${outputDir}/*`)
    .pipe(scp2({ ...scpConfig, host: 'jnofin115a' }))
    .on('error', (err) => {
      console.log(err);
    });
}

/* ssh copy all files in dest/ --> Finesse 12.x server */
function scp12() {
  return src(`${outputDir}/*`)
    .pipe(scp2({ ...scpConfig, host: 'jnofin125a' }))
    .on('error', (err) => {
      console.log(err);
    });
}

/* define task to perform SCP to 10.x followed by 11.x in series */
const upload = series(scp10, scp11);

/* define task to perform SCP to 12.x */
const upload12 = series(scp12);


/**
 * watch the scripts source, and whenever anything changes then execute
 * 'scripts' task followed by 'scp' which will mean all changes to output
 * files will be uploaded to Finesse.
 */
function watchFnc() {
  process.env.IS_GULP_WATCH = 'true';
  /* scripts --> dest/ */
  watch(
    paths.scripts.src,
    { ignoreInitial: false },
    series(['scripts'])
  );

  /* styles --> dest/ */
  watch(
    paths.styles.src,
    { ignoreInitial: false },
    series(['styles'])
  );

  /* assets --> dest/ */
  watch(
    paths.assets.src,
    { ignoreInitial: false },
    series(['copyassets'])
  );

  /* dest/ --> finesse server(s) with 2 second delay before copying */
  watch(
    `${outputDir}/**`,
    { ignoreInitial: false, delay: 2000 },
    series(['upload'])
  );
}


function clean() {
  // You can use multiple globbing patterns as you would with `gulp.src`,
  // for example if you are using del 2.0 or above, return its promise
  return del([outputDir, tempDir]);
}

/* copy non-transpiled assets to dest folder and make rw */
function copyassets() {
  return src(paths.assets.src, { since: lastRun(copyassets) })
    .pipe(chmod(666))
    .pipe(dest(paths.assets.dest));
}

/* convert .less --> .css into dest folder and make rw */
function styles() {
  return src(paths.styles.src, { since: lastRun(styles) })
    .pipe(less())
    .pipe(chmod(666))
    .pipe(dest(paths.styles.dest));
}

function init(cfg) {
  scpConfig.dest = cfg.scpdest;
  paths.scripts.src = cfg.scriptsSrc;
  webpackScripts = cfg.webpackScripts;
  copyScripts = cfg.copyScripts;
  /* optionals */
  // eslint-disable-next-line no-import-assign
  webpackConfig = cfg.webpackConfig || webpackConfig;
  // eslint-disable-next-line no-import-assign
  babelConfig = cfg.babelConfig || babelConfig;
  packageJson = cfg.packageJson;
  if (!packageJson) {
    throw new Error(`gulpfile-gadget-typescript:init() requires packageJson property. See that your LOCAL gulpfile.js passes in an object representing the package.json file`);
  }
}

/* Specify if tasks run in series or parallel using `gulp.series` and `gulp.parallel` */
const compile = series(copyassets, lint, parallel(styles, scripts));
const all = series(copyassets, lint, parallel(styles, scripts), upload);
const all12 = series(copyassets, lint, parallel(styles, scripts), upload12);

export {
  webpackConfig,
  babelConfig,
  paths,
  scpConfig,
  clean,
  copyassets,
  styles,
  scripts,
  compile,
  lint,
  upload,
  upload12,
  watchFnc,
  all,
  all12,
  init
};
